DPDK patches and discussions
 help / color / mirror / Atom feed
From: Robin Jarry <rjarry@redhat.com>
To: dev@dpdk.org
Cc: Robin Jarry <rjarry@redhat.com>, Olivier Matz <olivier.matz@6wind.com>
Subject: [PATCH] usertools/pmdinfo: remove dependency to ldd
Date: Thu, 13 Oct 2022 15:41:25 +0200	[thread overview]
Message-ID: <20221013134125.448437-1-rjarry@redhat.com> (raw)

Some environments (buildroot) do not have the ldd utility installed by
default. However, ldd is often only a wrapper shell script that actually
checks that the arguments are valid ELF files and executes them with
the LD_TRACE_LOADED_OBJECTS=1 variable set in the environment.

Since ld.so is the actual ELF interpreter which is loaded first when
executing a program, executing any dynamic ELF program/library with that
variable set will cause all dependent dynamic libraries to be printed
and ld.so will exit before even running main.

Excerpt from ld.so(7) man page:

  LD_TRACE_LOADED_OBJECTS
    If set (to any value), causes the program to list its dynamic
    dependencies, as if run by ldd(1), instead of running normally.

Change dpdk-pmdinfo.py to actually "execute" the files provided on the
command line with LD_TRACE_LOADED_OBJECTS=1 set. Ensure that the files
are valid dynamically executable ELF programs to avoid obscure and
confusing errors.

Reported-by: Olivier Matz <olivier.matz@6wind.com>
Signed-off-by: Robin Jarry <rjarry@redhat.com>
---
 doc/guides/tools/pmdinfo.rst |  4 ++--
 usertools/dpdk-pmdinfo.py    | 34 +++++++++++++++++++++++-----------
 2 files changed, 25 insertions(+), 13 deletions(-)

diff --git a/doc/guides/tools/pmdinfo.rst b/doc/guides/tools/pmdinfo.rst
index a9217de4eef2..1406b9c442eb 100644
--- a/doc/guides/tools/pmdinfo.rst
+++ b/doc/guides/tools/pmdinfo.rst
@@ -37,8 +37,8 @@ Arguments
 
 .. option:: ELF_FILE
 
-   DPDK application binary or dynamic library.
-   Any linked ``librte_*.so`` library (as reported by ``ldd``) will also be analyzed.
+   Executable DPDK application binary or dynamic library.
+   Any linked ``librte_*.so`` library (as reported by ``ld.so``) will also be analyzed.
    Can be specified multiple times.
 
 Environment Variables
diff --git a/usertools/dpdk-pmdinfo.py b/usertools/dpdk-pmdinfo.py
index 67d023a04711..01bb90666bcc 100755
--- a/usertools/dpdk-pmdinfo.py
+++ b/usertools/dpdk-pmdinfo.py
@@ -97,9 +97,9 @@ def parse_args() -> argparse.Namespace:
         "elf_files",
         metavar="ELF_FILE",
         nargs="+",
-        type=existing_file,
+        type=executable_elf_file,
         help="""
         DPDK application binary or dynamic library.
         """,
     )
     return parser.parse_args()
@@ -180,14 +180,24 @@ def get_plugin_libs(binaries: Iterable[Path]) -> Iterator[Path]:
 
 
 # ----------------------------------------------------------------------------
-def existing_file(value: str) -> Path:
+def executable_elf_file(value: str) -> Path:
     """
-    Argparse type= callback to ensure an argument points to a valid file path.
+    Argparse type= callback to ensure an argument points to a valid ELF file
+    path which can be executed.
     """
-    path = Path(value)
-    if not path.is_file():
-        raise argparse.ArgumentTypeError(f"{value}: No such file")
-    return path
+    try:
+        with open(value, "rb") as f:
+            elf = ELFFile(f)
+            if elf.header.e_type not in ("ET_DYN", "ET_EXEC"):
+                raise ELFError(f"unknown type: {elf.header.e_type!r}")
+        if not os.access(value, os.X_OK):
+            raise OSError("is not executable")
+    except ELFError as e:
+        raise argparse.ArgumentTypeError(f"{value}: invalid ELF: {e}") from e
+    except OSError as e:
+        raise argparse.ArgumentTypeError(f"{value}: {e}") from e
+
+    return Path(value)
 
 
 # ----------------------------------------------------------------------------
@@ -270,7 +280,7 @@ def get_elf_strings(path: Path, section: str, prefix: str) -> Iterator[str]:
 
 
 # ----------------------------------------------------------------------------
-LDD_LIB_RE = re.compile(
+LOADED_OBJECT_RE = re.compile(
     r"""
     ^                  # beginning of line
     \t                 # tab
@@ -290,14 +300,16 @@ def get_needed_libs(path: Path) -> Iterator[Path]:
     """
     Extract the dynamic library dependencies from an ELF executable.
     """
+    env = os.environ.copy()
+    env["LD_TRACE_LOADED_OBJECTS"] = "1"
     with subprocess.Popen(
-        ["ldd", str(path)], stdout=subprocess.PIPE, stderr=subprocess.PIPE
+        [str(path)], stdout=subprocess.PIPE, stderr=subprocess.PIPE, env=env
     ) as proc:
         out, err = proc.communicate()
         if proc.returncode != 0:
             err = err.decode("utf-8").splitlines()[-1].strip()
             raise Exception(f"cannot read ELF file: {err}")
-        for match in LDD_LIB_RE.finditer(out.decode("utf-8")):
+        for match in LOADED_OBJECT_RE.finditer(out.decode("utf-8")):
             libname, libpath = match.groups()
             if libname.startswith("librte_"):
                 libpath = Path(libpath)
@@ -97,9 +97,9 @@ def parse_args() -> argparse.Namespace:
         help="""
-        DPDK application binary or dynamic library.
+        Executable DPDK application binary or dynamic library.
         """,
     )
-- 
2.37.3


             reply	other threads:[~2022-10-13 13:41 UTC|newest]

Thread overview: 3+ messages / expand[flat|nested]  mbox.gz  Atom feed  top
2022-10-13 13:41 Robin Jarry [this message]
2022-10-14 14:02 ` Olivier Matz
2022-10-18  7:40   ` Olivier Matz

Reply instructions:

You may reply publicly to this message via plain-text email
using any one of the following methods:

* Save the following mbox file, import it into your mail client,
  and reply-to-all from there: mbox

  Avoid top-posting and favor interleaved quoting:
  https://en.wikipedia.org/wiki/Posting_style#Interleaved_style

* Reply using the --to, --cc, and --in-reply-to
  switches of git-send-email(1):

  git send-email \
    --in-reply-to=20221013134125.448437-1-rjarry@redhat.com \
    --to=rjarry@redhat.com \
    --cc=dev@dpdk.org \
    --cc=olivier.matz@6wind.com \
    /path/to/YOUR_REPLY

  https://kernel.org/pub/software/scm/git/docs/git-send-email.html

* If your mail client supports setting the In-Reply-To header
  via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line before the message body.
This is a public inbox, see mirroring instructions
for how to clone and mirror all data and code used for this inbox;
as well as URLs for NNTP newsgroup(s).